clear all
*! 3jan2017  Meru Bhanot (meru@uchicago.edu) and Michael Stepner (stepner@mit.edu)

* Modified by André Anundsen 10.08.2021
* imports 2019 U.S. CBSA shapefile into Stata format
/*******************************

** INPUT FILES ** 
- US_cbsa_2019.dbf
- US_cbsa_2019.shp
- US_state.dbf 
- US_state.shp
	
The shp-files have been	processed by André Anundsen using mapshaper.org to reduce its resolution to 0.2%
*******************************/


*** Step 0: Initialize

* Check if run using -project-
return clear
capture project, doinfo
if (_rc==0 & !mi(r(pname))) global root `r(pdir)'  // run using -project-
else {  // running directly

global root "$dataPath/Maps"
	* Disable project (since running do-files directly)
	cap program drop project
	program define project
		di "Project is disabled, skipping project command. (To re-enable, run -{stata program drop project}-)"
	end
	
}

* Specify subdirectories
global raw "$root/raw_data/cbsa2019"
global out "$root/constructed/cbsa2019"

* Add utility programs to path
adopath ++ "$root/util"

* Tell -project- that we use -save12-
project, original("$root/util/save12.ado")

* Create a temporary directory
cap mkdir "$root/temp"

*** Step 1: Process state outline shapefile

* Convert from shapefile to dta
shp2dta using "$raw/US_state_2019.shp", ///
	database("$root/temp/CBSA_stateborders_database.dta") ///
	coordinates("$root/temp/CBSA_stateborders_coords.dta") ///
	replace

* Reshape state outline shapefile

project, original("$root/util/reshape_us_CBSAstateoutline.ado")

reshape_us_CBSAstateoutline using "$root/temp/CBSA_stateborders_coords.dta", ///
	save_coords("$out/cbsa2019_stateborders.dta") ///
	save_shifts("$root/temp/CBSA_AlaskaHawaiiShifts.dta")
	
project, creates("$out/cbsa2019_stateborders.dta")

* Create state outline shapefile without AK or HI

use "$out/cbsa2019_stateborders.dta", clear
drop if !missing(sname)
drop sname 
save12 "${out}/cbsa2019_stateborders_noAKHI.dta", replace
project, creates("${out}/cbsa2019_stateborders_noAKHI.dta")

*** Step 3: Process CBSA shapefile

** Step 3.1: Unzip & convert shape file to dta
project, original("$raw/US_cbsa_2019.shp")
project, relies_on("$raw/US_cbsa_2019.dbf")

shp2dta using "$raw/US_cbsa_2019.shp", ///
	database("$root/temp/cbsa2019_database.dta") ///
	coordinates("$root/temp/cbsa2019_coords.dta") ///
	replace

** Step 3.2: Clean database
use "$root/temp/cbsa2019_database.dta", clear
gen statefp = 15 if regexm(NAME,", HI")
replace statefp = 2 if regexm(NAME,", AK")
drop if regexm(NAME,", PR") // Drop Puerto Rico

keep CBSAFP _ID statefp
rename CBSAFP cbsa2019
destring cbsa2019, replace
recast double cbsa2019

save12 "${out}/cbsa2019_database.dta", replace
project, creates("${out}/cbsa2019_database.dta")

** Step 3.3: Fix Coordinates
use "$root/temp/cbsa2019_coords.dta", clear
merge m:1 _ID using "$out/cbsa2019_database.dta", assert(1 3) keep(3) nogen

project, original("$root/util/reshape_us_CBSA.ado") preserve
reshape_us_CBSA using "$root/temp/CBSA_AlaskaHawaiiShifts.dta" // Rearrange Hawaii and Alaska

sort _ID, stable
keep _ID _X _Y

save12 "$out/cbsa2019_coords.dta", replace


project, creates("$out/cbsa2019_coords.dta")

** Step 3.4: Clean up temporary files
rm "$root/temp/CBSA_AlaskaHawaiiShifts.dta"
rm "$root/temp/CBSA_stateborders_coords.dta"
rm "$root/temp/CBSA_stateborders_database.dta"
rm "$root/temp/cbsa2019_coords.dta"
rm "$root/temp/cbsa2019_database.dta"
cap rmdir "$root/temp"

clear all

use "$out/cbsa2019_stateborders.dta", clear
bysort _ID : gen ordering = _n
drop if sname == "AK" | sname == "HI"
egen _ID2 = group(_ID)
save "$out/cbsa2019_stateborders.dta", replace

use "$out/cbsa2019_coords.dta", clear
bysort _ID : gen ordering = _n
save "$out/cbsa2019_coords.dta", replace

